$:.unshift File.dirname(__FILE__)

require 'minitest/autorun'

require 'array_extensions_test'
require 'ea_instantiator_test'
require 'ecore_self_test'
require 'environment_test'
require 'metamodel_builder_test'
require 'metamodel_roundtrip_test'
require 'output_handler_test'
require 'template_language_test'
require 'transformer_test'
require 'xml_instantiator_test'
require 'ea_serializer_test'
require 'model_builder_test'
require 'method_delegation_test'
require 'json_test'
require 'reference_resolver_test'
require 'qualified_name_resolver_test'
require 'metamodel_order_test'
require 'metamodel_from_ecore_test'
require 'util_test'
require 'model_fragment_test'
require 'qualified_name_provider_test'
require 'ecore_to_ruby_test'

